@@ -1,7 +1,7 @@
 ## code for fast REML computation. key feature is that first and 
 ## second derivatives come at no increase in leading order 
 ## computational cost, relative to evaluation! 
-## (c) Simon N. Wood, 2010-2012
+## (c) Simon N. Wood, 2010-2014
 
 Sl.setup <- function(G) {
 ## Sets up a list representing a block diagonal penalty matrix.
@@ -58,20 +58,11 @@ Sl.setup <- function(G) {
   } ## finished paraPen
 
   ## now work through the smooths....
-  for (i in 1:length(G$smooth)) {
+  if (length(G$smooth)) for (i in 1:length(G$smooth)) {
 
     if (!is.null(G$smooth[[i]]$fixed)&&G$smooth[[i]]$fixed) m <- 0 else
     m <- length(G$smooth[[i]]$S)
 
-   # if (m>0&&!is.null(G$smooth[[i]]$fx)) { ## drop any fixed 
-   #   for (j in m:1) {
-   #     if (G$smooth[[i]]$fx[j]==TRUE) { 
-   #       G$smooth[[i]]$S[[j]] <- NULL
-   #       m <- m - 1
-   #     }
-   #   }
-   # }
-
     if (m>0) {
       Sl[[b]] <- list()
       Sl[[b]]$start <- G$smooth[[i]]$first.para
@@ -116,7 +107,6 @@ Sl.setup <- function(G) {
           b <- b + 1
         }
       } else { ## not possible to split
-        #Sl[[b]]$rank <- nrow(G$smooth[[i]]$S[[1]]) - G$smooth[[i]]$null.space.dim+1 
         Sl[[b]]$S <- G$smooth[[i]]$S
       
       } ## additive block finished
@@ -129,6 +119,8 @@ Sl.setup <- function(G) {
   ## Singletons need to be transformed to identity penalties, while 
   ## multiples need to be projected into total penalty range space. 
 
+  if (length(Sl)==0) return(Sl) ## nothing to do
+
   np <- ncol(G$X)
   E <- matrix(0,np,np) ## well scaled square root penalty
   lambda <- rep(0,0)
@@ -155,7 +147,6 @@ Sl.setup <- function(G) {
         Sl[[b]]$D <- t(D*t(U)) ## D <- U%*%diag(D)
         Sl[[b]]$Di <- t(U)/D
         ## so if X is smooth model matrix X%*%D is re-parameterized form 
-        ## Sl[[b]]$D <- D; 
         Sl[[b]]$ind <- ind
       }
       ## add penalty square root into E  
@@ -179,8 +170,7 @@ Sl.setup <- function(G) {
       }
       Sl[[b]]$ind <- rep(FALSE,ncol(U))
       Sl[[b]]$ind[ind] <- TRUE ## index penalized within sub-range
-      ## Sl[[b]]$stop = Sl[[b]]$start + Sl[[b]]$rank - 1
-      ## Sl[[b]]$ind <- ind ## IS THIS NEEDED?? (if so should be logical!!)
+     
       ## now compute well scaled sqrt
       S.norm <- norm(Sl[[b]]$S[[1]])
       St <- Sl[[b]]$S[[1]]/S.norm
@@ -206,6 +196,7 @@ Sl.initial.repara <- function(Sl,X,inverse=FALSE,both.sides=TRUE,cov=TRUE) {
 ## or, if inverse==TRUE, to apply inverse re-para to parameter vector 
 ## or cov matrix. if inverse is TRUE and both.sides=FALSE then 
 ## re-para only applied to rhs, as appropriate for a choleski factor.
+  if (length(Sl)==0) return(X) ## nothing to do
   if (inverse) { ## apply inverse re-para
     if (is.matrix(X)) { 
       if (cov) { ## then it's a covariance matrix
@@ -313,7 +304,7 @@ ldetS <- function(Sl,rho,fixed,np,root=FALSE) {
       }      
     } ## end of multi-S block branch
   } ## end of block loop
-  if (root) E <- E[rowSums(abs(E))!=0,] ## drop zero rows.
+  if (root) E <- E[rowSums(abs(E))!=0,,drop=FALSE] ## drop zero rows.
   list(ldetS=ldS,ldet1=d1.ldS,ldet2=d2.ldS,Sl=Sl,rp=rp,E=E)
 } ## end ldetS
 
@@ -405,7 +396,7 @@ Sl.mult <- function(Sl,A,k = 0,full=TRUE) {
   A
 } ## end Sl.mult
 
-Sl.termMult <- function(Sl,A,full=FALSE) {
+Sl.termMult <- function(Sl,A,full=FALSE,nt=1) {
 ## returns a list containing the product of each element S of Sl
 ## with A. If full==TRUE then the results include the zero rows
 ## otherwise these are stripped out, but in that case each element 
@@ -435,12 +426,16 @@ Sl.termMult <- function(Sl,A,full=FALSE) {
         k <- k + 1
         if (full) { ## return answer with all zeroes in place
           B <- A*0
-          if (Amat) B[ind,] <- Sl[[b]]$Srp[[i]]%*%A[ind,,drop=FALSE] else  
-                    B[ind] <- Sl[[b]]$Srp[[i]]%*%A[ind]
+          if (Amat) { 
+            B[ind,] <- if (nt==1) Sl[[b]]$Srp[[i]]%*%A[ind,,drop=FALSE] else 
+                       pmmult(Sl[[b]]$Srp[[i]],A[ind,,drop=FALSE],nt=nt) 
+          } else B[ind] <- Sl[[b]]$Srp[[i]]%*%A[ind]
           SA[[k]] <- B
         } else { ## strip zero rows from answer
-          if (Amat) SA[[k]] <- Sl[[b]]$Srp[[i]]%*%A[ind,,drop=FALSE] else
-                    SA[[k]] <- as.numeric(Sl[[b]]$Srp[[i]]%*%A[ind])
+          if (Amat) {
+            SA[[k]] <- if (nt==1) Sl[[b]]$Srp[[i]]%*%A[ind,,drop=FALSE] else
+                       pmmult(Sl[[b]]$Srp[[i]],A[ind,,drop=FALSE],nt=nt)
+          } else SA[[k]] <- as.numeric(Sl[[b]]$Srp[[i]]%*%A[ind])
           attr(SA[[k]],"ind") <- ind
         }
       } ## end of S loop for block b
@@ -449,10 +444,10 @@ Sl.termMult <- function(Sl,A,full=FALSE) {
   SA
 } ## end Sl.termMult
 
-d.detXXS <- function(Sl,PP) {
+d.detXXS <- function(Sl,PP,nt=1) {
 ## function to obtain derivatives of log |X'X+S| given unpivoted PP' where 
 ## P is inverse of R from the QR of the augmented model matrix. 
-  SPP <- Sl.termMult(Sl,PP,full=FALSE) ## SPP[[k]] is S_k PP'
+  SPP <- Sl.termMult(Sl,PP,full=FALSE,nt=nt) ## SPP[[k]] is S_k PP'
   nd <- length(SPP)
   d1 <- rep(0,nd);d2 <- matrix(0,nd,nd) 
   for (i in 1:nd) { 
@@ -500,10 +495,10 @@ Sl.ift <- function(Sl,R,X,y,beta,piv,rp) {
                                 (k==j)*sum(beta*Skb[[k]])
     }
   }
-  list(rss =sum(rsd^2),bSb=sum(beta*Sb),rss1=rss1,bSb1=bSb1,rss2=rss2,bSb2=bSb2)
+  list(rss =sum(rsd^2),bSb=sum(beta*Sb),rss1=rss1,bSb1=bSb1,rss2=rss2,bSb2=bSb2,d1b=db)
 } ## end Sl.ift
 
-Sl.fit <- function(Sl,X,y,rho,fixed,log.phi=0,phi.fixed=TRUE,rss.extra=0,nobs=NULL,Mp=0) {
+Sl.fit <- function(Sl,X,y,rho,fixed,log.phi=0,phi.fixed=TRUE,rss.extra=0,nobs=NULL,Mp=0,nt=1) {
 ## fits penalized regression model with model matrix X and 
 ## initialised block diagonal penalty Sl to data in y, given 
 ## log smoothing parameters rho. 
@@ -516,8 +511,8 @@ Sl.fit <- function(Sl,X,y,rho,fixed,log.phi=0,phi.fixed=TRUE,rss.extra=0,nobs=NU
   ldS <- ldetS(Sl,rho,fixed,np,root=TRUE)
   ## apply resulting stable re-parameterization to X...
   X <- Sl.repara(ldS$rp,X)
-  ## get pivoted QR decomp of augmented model matrix
-  qrx <- qr(rbind(X,ldS$E),LAPACK=TRUE)
+  ## get pivoted QR decomp of augmented model matrix (in parallel if nt>1)
+  qrx <- if (nt>1) pqr2(rbind(X,ldS$E),nt=nt) else qr(rbind(X,ldS$E),LAPACK=TRUE)
   rp <- qrx$pivot;rp[rp] <- 1:np ## reverse pivot vector
   ## find pivoted \hat beta...
   R <- qr.R(qrx)
@@ -527,10 +522,12 @@ Sl.fit <- function(Sl,X,y,rho,fixed,log.phi=0,phi.fixed=TRUE,rss.extra=0,nobs=NU
   ## get component derivatives based on IFT...
   dift <- Sl.ift(ldS$Sl,R,X,y,beta,qrx$pivot,rp)
   ## and the derivatives of log|X'X+S|...
-  P <- backsolve(R,diag(np))[rp,] ## invert R and row unpivot
-  PP <- tcrossprod(P) ## PP'
+  P <- pbsi(R,nt=nt,copy=TRUE) ## invert R 
+  ## P <- backsolve(R,diag(np))[rp,] ## invert R and row unpivot
+  ## crossprod and unpivot (don't unpivot if unpivoting P above)
+  PP <- if (nt==1) tcrossprod(P)[rp,rp] else pRRt(P,nt)[rp,rp] ## PP'
   ldetXXS <- 2*sum(log(abs(diag(R)))) ## log|X'X+S|
-  dXXS <- d.detXXS(ldS$Sl,PP) ## derivs of log|X'X+S|
+  dXXS <- d.detXXS(ldS$Sl,PP,nt=nt) ## derivs of log|X'X+S|
   ## all ingredients are now in place to form REML score and 
   ## its derivatives....
   reml <- (rss.bSb/phi + (nobs-Mp)*log(2*pi*phi) +
@@ -552,11 +549,11 @@ Sl.fit <- function(Sl,X,y,rho,fixed,log.phi=0,phi.fixed=TRUE,rss.extra=0,nobs=NU
   #list(reml=dift$rss,reml1=dift$rss1,reml2=dift$rss2)
   #list(reml=dift$bSb,reml1=dift$bSb1,reml2=dift$bSb2) 
   list(reml=as.numeric(reml),reml1=reml1,reml2=reml2,beta=beta[rp],PP=PP,
-       rp=ldS$rp,rss=dift$rss+rss.extra,nobs=nobs)
-}
+       rp=ldS$rp,rss=dift$rss+rss.extra,nobs=nobs,d1b=dift$d1b)
+} ## Sl.fit
 
 fast.REML.fit <- function(Sl,X,y,rho,L=NULL,rho.0=NULL,log.phi=0,phi.fixed=TRUE,
-                 rss.extra=0,nobs=NULL,Mp=0,conv.tol=.Machine$double.eps^.5) {
+                 rss.extra=0,nobs=NULL,Mp=0,conv.tol=.Machine$double.eps^.5,nt=1) {
 ## estimates log smoothing parameters rho, by optimizing fast REML 
 ## using Newton's method. On input Sl is a block diagonal penalty 
 ## structure produced by Sl.setup, while X is a model matrix 
@@ -569,7 +566,7 @@ fast.REML.fit <- function(Sl,X,y,rho,L=NULL,rho.0=NULL,log.phi=0,phi.fixed=TRUE,
   if (is.null(nobs)) nobs <- nrow(X)
   np <- ncol(X)
   if (nrow(X) > np) { ## might as well do an initial QR step
-    qrx <- qr(X,LAPACK=TRUE)
+    qrx <- if (nt>1) pqr2(X,nt=nt) else qr(X,LAPACK=TRUE)
     rp <- qrx$pivot
     rp[rp] <- 1:np
     X <- qr.R(qrx)[,rp]
@@ -589,7 +586,7 @@ fast.REML.fit <- function(Sl,X,y,rho,L=NULL,rho.0=NULL,log.phi=0,phi.fixed=TRUE,
   fixed <- rep(FALSE,nrow(L))
  
   
-  best <- Sl.fit(Sl,X,y,L%*%rho+rho.0,fixed,log.phi,phi.fixed,rss.extra,nobs,Mp)
+  best <- Sl.fit(Sl,X,y,L%*%rho+rho.0,fixed,log.phi,phi.fixed,rss.extra,nobs,Mp,nt=nt)
   ## get a typical scale for the reml score... 
   reml.scale <- abs(best$reml) + best$rss/best$nobs
  
@@ -633,12 +630,12 @@ fast.REML.fit <- function(Sl,X,y,rho,L=NULL,rho.0=NULL,log.phi=0,phi.fixed=TRUE,
     step[uconv.ind] <- uc.step ## step includes converged
     ## try out the step...
     rho1 <- L%*%(rho + step)+rho.0; if (!phi.fixed) log.phi <- rho1[nr+1]
-    trial <- Sl.fit(Sl,X,y,rho1[1:nr],fixed,log.phi,phi.fixed,rss.extra,nobs,Mp)
+    trial <- Sl.fit(Sl,X,y,rho1[1:nr],fixed,log.phi,phi.fixed,rss.extra,nobs,Mp,nt=nt)
     k <- 0
     while (trial$reml>best$reml && k<35) { ## step half until improvement
       step <- step/2;k <- k + 1
       rho1 <- L%*%(rho + step)+rho.0; if (!phi.fixed) log.phi <- rho1[nr+1]
-      trial <- Sl.fit(Sl,X,y,rho1[1:nr],fixed,log.phi,phi.fixed,rss.extra,nobs,Mp)
+      trial <- Sl.fit(Sl,X,y,rho1[1:nr],fixed,log.phi,phi.fixed,rss.extra,nobs,Mp,nt=nt)
     }
     if (k==35 && trial$reml>best$reml) { ## step has failed
       step.failed <- TRUE
@@ -676,7 +673,7 @@ fast.REML.fit <- function(Sl,X,y,rho,L=NULL,rho.0=NULL,log.phi=0,phi.fixed=TRUE,
   best ## return the best fit (note that it will need post-processing to be useable)
 } ## end fast.REML.fit
 
-ident.test <- function(X,E) {
+ident.test <- function(X,E,nt=1) {
 ## routine to identify structurally un-identifiable coefficients
 ## for model with model matrix X and scaled sqrt penalty matrix E
 ## lambda is smoothing parameter vector corresponding to E, 
@@ -688,7 +685,7 @@ ident.test <- function(X,E) {
 ## then beta.full[undrop] <- beta, is the full, zero padded 
 ## coeff vector, with dropped coefs re-nstated as zeroes. 
   Xnorm <- norm(X,type="F")
-  qrx <- qr(rbind(X/Xnorm,E),LAPACK=TRUE) ## pivoted QR
+  qrx <- if (nt>1) pqr2(rbind(X/Xnorm,E),nt=nt) else qr(rbind(X/Xnorm,E),LAPACK=TRUE) ## pivoted QR
   rank <- Rrank(qr.R(qrx),tol=.Machine$double.eps^.75)
   drop <- qrx$pivot[-(1:rank)] ## index of un-identifiable coefs
   undrop <- 1:ncol(X) 
@@ -754,13 +751,13 @@ Sl.drop <- function(Sl,drop,np) {
   Sl
 } ## Sl.drop
 
-Sl.Xprep <- function(Sl,X) { 
+Sl.Xprep <- function(Sl,X,nt=1) { 
 ## Sl is block diag object from Sl.setup, X is a model matrix
 ## this routine applies preliminary Sl transformations to X
 ## tests for structural identifibility problems and drops
 ## un-identifiable parameters.
   X <- Sl.initial.repara(Sl,X) ## apply re-para used in Sl to X
-  id <- ident.test(X,attr(Sl,"E")) ## deal with structural identifiability
+  id <- ident.test(X,attr(Sl,"E"),nt=nt) ## deal with structural identifiability
   ## id contains drop, undrop, lambda
   if (length(id$drop)>0) { ## then there is something to do here 
     Sl <- Sl.drop(Sl,id$drop,ncol(X)) ## drop unidentifiable from Sl
@@ -782,7 +779,12 @@ Sl.postproc <- function(Sl,fit,undrop,X0,cov=FALSE,scale = -1) {
   beta <- rep(0,np)
   beta[undrop] <- Sl.repara(fit$rp,fit$beta,inverse=TRUE)
   beta <- Sl.initial.repara(Sl,beta,inverse=TRUE)
-  if (cov) {
+ 
+  if (cov) { 
+    d1b <- matrix(0,np,ncol(fit$d1b))
+    ## following construction a bit ugly due to Sl.repara assumptions...
+    d1b[undrop,] <- t(Sl.repara(fit$rp,t(fit$d1b),inverse=TRUE,both.sides=FALSE))
+    for (i in 1:ncol(d1b)) d1b[,i] <- Sl.initial.repara(Sl,as.numeric(d1b[,i]),inverse=TRUE) ## d beta / d rho matrix
     PP <- matrix(0,np,np)
     PP[undrop,undrop] <-  Sl.repara(fit$rp,fit$PP,inverse=TRUE)
     PP <- Sl.initial.repara(Sl,PP,inverse=TRUE)
@@ -796,10 +798,20 @@ Sl.postproc <- function(Sl,fit,undrop,X0,cov=FALSE,scale = -1) {
     ## edf <- rowSums(PP*crossprod(X0)) ## diag(PP%*%(t(X0)%*%X0))
     if (scale<=0) scale <- fit$rss/(fit$nobs - sum(edf))
     Vp <- PP * scale ## cov matrix
+    ## sp uncertainty correction... 
+    M <- ncol(d1b) 
+    ev <- eigen(fit$outer.info$hess,symmetric=TRUE)
+    ind <- ev$values <= 0
+    ev$values[ind] <- 0;ev$values[!ind] <- 1/sqrt(ev$values[!ind])
+    rV <- (ev$values*t(ev$vectors))[,1:M]
+    Vc <- crossprod(rV%*%t(d1b))
+    Vc <- Vp + Vc  ## Bayesian cov matrix with sp uncertainty
+    edf2 <- rowSums(Vc*crossprod(X0))/scale
+
     ##bias <- as.numeric(beta-F%*%beta) ## estimate of smoothing bias in beta
-    return(list(beta=beta,Vp=Vp,Ve=F%*%Vp,edf=edf,edf1=edf1,hat=hat,F=F))
+    return(list(beta=beta,Vp=Vp,Vc=Vc,Ve=F%*%Vp,edf=edf,edf1=edf1,edf2=edf2,hat=hat,F=F))
   } else return(list(beta=beta))
-}
+} ## Sl.postproc
 
 
 ## USEAGE SEQUENCE:
